@@ -11,14 +11,19 @@ DOCUMENTATION = '''
 ---
 module: ec2_metadata_facts
 version_added: 1.0.0
-short_description: Gathers facts (instance metadata) about remote hosts within ec2
+short_description: gathers facts (instance metadata) about remote hosts within EC2
 author:
     - Silviu Dicu (@silviud)
     - Vinay Dandekar (@roadmapper)
 description:
-    - This module fetches data from the instance metadata endpoint in ec2 as per
+    - This module fetches data from the instance metadata endpoint in EC2 as per
       U(https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html).
     - The module must be called from within the EC2 instance itself.
+    - The module is configured to utilize the session oriented Instance Metadata Service v2 (IMDSv2)
+      U(https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-service.html).
+    - If the HttpEndpoint parameter
+      U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceMetadataOptions.html#API_ModifyInstanceMetadataOptions_RequestParameters)
+      is set to disabled for the EC2 instance, the module will return an error while retrieving a session token.
 notes:
     - Parameters to filter on ec2_metadata_facts may be added later.
 '''
@@ -114,7 +119,7 @@ ansible_facts:
         ansible_ec2_iam_info_instanceprofilearn:
             description: The IAM instance profile ARN.
             type: str
-            sample: "arn:aws:iam::<account id>:instance-profile/<role name>"
+            sample: "arn:aws:iam::<account id>:instance-profile/role_name"
         ansible_ec2_iam_info_instanceprofileid:
             description: IAM instance profile ID.
             type: str
@@ -127,37 +132,37 @@ ansible_facts:
             description: IAM instance role.
             type: str
             sample: "role_name"
-        ansible_ec2_iam_security_credentials_<role name>:
+        ansible_ec2_iam_security_credentials_role_name:
             description:
                 - If there is an IAM role associated with the instance, role-name is the name of the role,
                   and role-name contains the temporary security credentials associated with the role. Otherwise, not present.
             type: str
             sample: ""
-        ansible_ec2_iam_security_credentials_<role name>_accesskeyid:
+        ansible_ec2_iam_security_credentials_role_name_accesskeyid:
             description: IAM role access key ID.
             type: str
             sample: ""
-        ansible_ec2_iam_security_credentials_<role name>_code:
+        ansible_ec2_iam_security_credentials_role_name_code:
             description: IAM code.
             type: str
             sample: "Success"
-        ansible_ec2_iam_security_credentials_<role name>_expiration:
+        ansible_ec2_iam_security_credentials_role_name_expiration:
             description: IAM role credentials expiration time.
             type: str
             sample: "2017-05-12T09:11:41Z"
-        ansible_ec2_iam_security_credentials_<role name>_lastupdated:
+        ansible_ec2_iam_security_credentials_role_name_lastupdated:
             description: IAM role last updated time.
             type: str
             sample: "2017-05-12T02:40:44Z"
-        ansible_ec2_iam_security_credentials_<role name>_secretaccesskey:
+        ansible_ec2_iam_security_credentials_role_name_secretaccesskey:
             description: IAM role secret access key.
             type: str
             sample: ""
-        ansible_ec2_iam_security_credentials_<role name>_token:
+        ansible_ec2_iam_security_credentials_role_name_token:
             description: IAM role token.
             type: str
             sample: ""
-        ansible_ec2_iam_security_credentials_<role name>_type:
+        ansible_ec2_iam_security_credentials_role_name_type:
             description: IAM role type.
             type: str
             sample: "AWS-HMAC"
@@ -243,8 +248,12 @@ ansible_facts:
             description: Data that can be used by other parties to verify its origin and authenticity.
             type: str
             sample: ""
+        ansible_ec2_instance_life_cycle:
+            description: The purchasing option of the instance.
+            type: str
+            sample: "on-demand"
         ansible_ec2_instance_type:
-            description: The type of instance.
+            description: The type of the instance.
             type: str
             sample: "m4.large"
         ansible_ec2_local_hostname:
@@ -266,90 +275,90 @@ ansible_facts:
             type: str
             sample: "00:11:22:33:44:55"
         ansible_ec2_metrics_vhostmd:
-            description: Metrics.
+            description: Metrics; no longer available.
             type: str
             sample: ""
-        ansible_ec2_network_interfaces_macs_<mac address>_device_number:
+        ansible_ec2_network_interfaces_macs_mac_address_device_number:
             description:
                 - The unique device number associated with that interface. The device number corresponds to the device name;
                   for example, a device-number of 2 is for the eth2 device.
                 - This category corresponds to the DeviceIndex and device-index fields that are used by the Amazon EC2 API and the EC2 commands for the AWS CLI.
             type: str
             sample: "0"
-        ansible_ec2_network_interfaces_macs_<mac address>_interface_id:
+        ansible_ec2_network_interfaces_macs_mac_address_interface_id:
             description: The elastic network interface ID.
             type: str
             sample: "eni-12345678"
-        ansible_ec2_network_interfaces_macs_<mac address>_ipv4_associations_<ip address>:
+        ansible_ec2_network_interfaces_macs_mac_address_ipv4_associations_ip_address:
             description: The private IPv4 addresses that are associated with each public-ip address and assigned to that interface.
             type: str
             sample: ""
-        ansible_ec2_network_interfaces_macs_<mac address>_ipv6s:
+        ansible_ec2_network_interfaces_macs_mac_address_ipv6s:
             description: The IPv6 addresses associated with the interface. Returned only for instances launched into a VPC.
             type: str
             sample: ""
-        ansible_ec2_network_interfaces_macs_<mac address>_local_hostname:
+        ansible_ec2_network_interfaces_macs_mac_address_local_hostname:
             description: The interface's local hostname.
             type: str
             sample: ""
-        ansible_ec2_network_interfaces_macs_<mac address>_local_ipv4s:
+        ansible_ec2_network_interfaces_macs_mac_address_local_ipv4s:
             description: The private IPv4 addresses associated with the interface.
             type: str
             sample: ""
-        ansible_ec2_network_interfaces_macs_<mac address>_mac:
+        ansible_ec2_network_interfaces_macs_mac_address_mac:
             description: The instance's MAC address.
             type: str
             sample: "00:11:22:33:44:55"
-        ansible_ec2_network_interfaces_macs_<mac address>_owner_id:
+        ansible_ec2_network_interfaces_macs_mac_address_owner_id:
             description:
                 - The ID of the owner of the network interface.
                 - In multiple-interface environments, an interface can be attached by a third party, such as Elastic Load Balancing.
                 - Traffic on an interface is always billed to the interface owner.
             type: str
             sample: "01234567890"
-        ansible_ec2_network_interfaces_macs_<mac address>_public_hostname:
+        ansible_ec2_network_interfaces_macs_mac_address_public_hostname:
             description:
                 - The interface's public DNS (IPv4). If the instance is in a VPC,
                   this category is only returned if the enableDnsHostnames attribute is set to true.
             type: str
             sample: "ec2-1-2-3-4.compute-1.amazonaws.com"
-        ansible_ec2_network_interfaces_macs_<mac address>_public_ipv4s:
+        ansible_ec2_network_interfaces_macs_mac_address_public_ipv4s:
             description: The Elastic IP addresses associated with the interface. There may be multiple IPv4 addresses on an instance.
             type: str
             sample: "1.2.3.4"
-        ansible_ec2_network_interfaces_macs_<mac address>_security_group_ids:
+        ansible_ec2_network_interfaces_macs_mac_address_security_group_ids:
             description: The IDs of the security groups to which the network interface belongs. Returned only for instances launched into a VPC.
             type: str
             sample: "sg-01234567,sg-01234568"
-        ansible_ec2_network_interfaces_macs_<mac address>_security_groups:
+        ansible_ec2_network_interfaces_macs_mac_address_security_groups:
             description: Security groups to which the network interface belongs. Returned only for instances launched into a VPC.
             type: str
             sample: "secgroup1,secgroup2"
-        ansible_ec2_network_interfaces_macs_<mac address>_subnet_id:
+        ansible_ec2_network_interfaces_macs_mac_address_subnet_id:
             description: The ID of the subnet in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: "subnet-01234567"
-        ansible_ec2_network_interfaces_macs_<mac address>_subnet_ipv4_cidr_block:
+        ansible_ec2_network_interfaces_macs_mac_address_subnet_ipv4_cidr_block:
             description: The IPv4 CIDR block of the subnet in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: "10.0.1.0/24"
-        ansible_ec2_network_interfaces_macs_<mac address>_subnet_ipv6_cidr_blocks:
+        ansible_ec2_network_interfaces_macs_mac_address_subnet_ipv6_cidr_blocks:
             description: The IPv6 CIDR block of the subnet in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: ""
-        ansible_ec2_network_interfaces_macs_<mac address>_vpc_id:
+        ansible_ec2_network_interfaces_macs_mac_address_vpc_id:
             description: The ID of the VPC in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: "vpc-0123456"
-        ansible_ec2_network_interfaces_macs_<mac address>_vpc_ipv4_cidr_block:
+        ansible_ec2_network_interfaces_macs_mac_address_vpc_ipv4_cidr_block:
             description: The IPv4 CIDR block of the VPC in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: "10.0.0.0/16"
-        ansible_ec2_network_interfaces_macs_<mac address>_vpc_ipv4_cidr_blocks:
+        ansible_ec2_network_interfaces_macs_mac_address_vpc_ipv4_cidr_blocks:
             description: The IPv4 CIDR block of the VPC in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: "10.0.0.0/16"
-        ansible_ec2_network_interfaces_macs_<mac address>_vpc_ipv6_cidr_blocks:
+        ansible_ec2_network_interfaces_macs_mac_address_vpc_ipv6_cidr_blocks:
             description: The IPv6 CIDR block of the VPC in which the interface resides. Returned only for instances launched into a VPC.
             type: str
             sample: ""
@@ -434,33 +443,41 @@ socket.setdefaulttimeout(5)
 
 
 class Ec2Metadata(object):
+    ec2_metadata_token_uri = 'http://169.254.169.254/latest/api/token'
     ec2_metadata_uri = 'http://169.254.169.254/latest/meta-data/'
     ec2_sshdata_uri = 'http://169.254.169.254/latest/meta-data/public-keys/0/openssh-key'
     ec2_userdata_uri = 'http://169.254.169.254/latest/user-data/'
     ec2_dynamicdata_uri = 'http://169.254.169.254/latest/dynamic/'
 
-    def __init__(self, module, ec2_metadata_uri=None, ec2_sshdata_uri=None, ec2_userdata_uri=None, ec2_dynamicdata_uri=None):
+    def __init__(self, module, ec2_metadata_token_uri=None, ec2_metadata_uri=None, ec2_sshdata_uri=None, ec2_userdata_uri=None, ec2_dynamicdata_uri=None):
         self.module = module
+        self.uri_token = ec2_metadata_token_uri or self.ec2_metadata_token_uri
         self.uri_meta = ec2_metadata_uri or self.ec2_metadata_uri
         self.uri_user = ec2_userdata_uri or self.ec2_userdata_uri
         self.uri_ssh = ec2_sshdata_uri or self.ec2_sshdata_uri
         self.uri_dynamic = ec2_dynamicdata_uri or self.ec2_dynamicdata_uri
         self._data = {}
+        self._token = None
         self._prefix = 'ansible_ec2_%s'
 
     def _fetch(self, url):
         encoded_url = quote(url, safe='%/:=&?~#+!$,;\'@()*[]')
-        response, info = fetch_url(self.module, encoded_url, force=True)
+        headers = {}
+        if self._token:
+            headers = {'X-aws-ec2-metadata-token': self._token}
+        response, info = fetch_url(self.module, encoded_url, headers=headers, force=True)
 
-        if info.get('status') not in (200, 404):
+        if info.get('status') in (401, 403):
+            self.module.fail_json(msg='Failed to retrieve metadata from AWS: {0}'.format(info['msg']), response=info)
+        elif info.get('status') not in (200, 404):
             time.sleep(3)
             # request went bad, retry once then raise
             self.module.warn('Retrying query to metadata service. First attempt failed: {0}'.format(info['msg']))
-            response, info = fetch_url(self.module, encoded_url, force=True)
+            response, info = fetch_url(self.module, encoded_url, headers=headers, force=True)
             if info.get('status') not in (200, 404):
                 # fail out now
                 self.module.fail_json(msg='Failed to retrieve metadata from AWS: {0}'.format(info['msg']), response=info)
-        if response:
+        if response and info['status'] < 400:
             data = response.read()
         else:
             data = None
@@ -525,7 +542,29 @@ class Ec2Metadata(object):
 
         return new_data
 
+    def fetch_session_token(self, uri_token):
+        """Used to get a session token for IMDSv2"""
+        headers = {'X-aws-ec2-metadata-token-ttl-seconds': '60'}
+        response, info = fetch_url(self.module, uri_token, method='PUT', headers=headers, force=True)
+
+        if info.get('status') == 403:
+            self.module.fail_json(msg='Failed to retrieve metadata token from AWS: {0}'.format(info['msg']), response=info)
+        elif info.get('status') not in (200, 404):
+            time.sleep(3)
+            # request went bad, retry once then raise
+            self.module.warn('Retrying query to metadata service. First attempt failed: {0}'.format(info['msg']))
+            response, info = fetch_url(self.module, uri_token, method='PUT', headers=headers, force=True)
+            if info.get('status') not in (200, 404):
+                # fail out now
+                self.module.fail_json(msg='Failed to retrieve metadata token from AWS: {0}'.format(info['msg']), response=info)
+        if response:
+            token_data = response.read()
+        else:
+            token_data = None
+        return to_text(token_data)
+
     def run(self):
+        self._token = self.fetch_session_token(self.uri_token)  # create session token for IMDS
         self.fetch(self.uri_meta)  # populate _data with metadata
         data = self._mangle_fields(self._data, self.uri_meta)
         data[self._prefix % 'user-data'] = self._fetch(self.uri_user)
