@@ -14,13 +14,13 @@ ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported
 
 DOCUMENTATION = '''
 ---
-module: tower_workflow_job_template
+module: workflow_job_template
 author: "John Westcott IV (@john-westcott-iv)"
-short_description: create, update, or destroy Ansible Tower workflow job templates.
+short_description: create, update, or destroy Automation Platform Controller workflow job templates.
 description:
-    - Create, update, or destroy Ansible Tower workflow job templates.
+    - Create, update, or destroy Automation Platform Controller workflow job templates.
     - Replaces the deprecated tower_workflow_template module.
-    - Use the tower_workflow_job_template_node after this, or use the schema paramater to build the workflow's graph
+    - Use workflow_job_template_node after this, or use the workflow_nodes parameter to build the workflow's graph
 options:
     name:
       description:
@@ -47,10 +47,16 @@ options:
       description:
         - Variables which will be made available to jobs ran inside the workflow.
       type: dict
-    execution_environment:
+    job_tags:
       description:
-        - Execution Environment to use for the WFJT.
+        - Comma separated list of the tags to use for the job template.
       type: str
+    ask_tags_on_launch:
+      description:
+        - Prompt user for job tags on launch.
+      type: bool
+      aliases:
+        - ask_tags
     organization:
       description:
         - Organization the workflow job template exists in.
@@ -89,6 +95,22 @@ options:
       description:
         - Prompt user for limit on launch of this workflow job template
       type: bool
+    ask_labels_on_launch:
+      description:
+        - Prompt user for labels on launch.
+      type: bool
+      aliases:
+        - ask_labels
+    ask_skip_tags_on_launch:
+      description:
+        - Prompt user for job tags to skip on launch.
+      type: bool
+      aliases:
+        - ask_skip_tags
+    skip_tags:
+      description:
+        - Comma separated list of the tags to skip for the job template.
+      type: str
     webhook_service:
       description:
         - Service that webhook requests will be accepted from
@@ -114,6 +136,7 @@ options:
     labels:
       description:
         - The labels applied to this job template
+        - Must be created with the labels module first. This will error if the label has not been created.
       type: list
       elements: str
     state:
@@ -141,13 +164,16 @@ options:
       elements: str
     notification_templates_approvals:
       description:
-        - list of notifications to send on start
+        - list of notifications to send on approval
       type: list
       elements: str
-    schema:
+    workflow_nodes:
       description:
         - A json list of nodes and their coresponding options. The following suboptions describe a single node.
       type: list
+      elements: dict
+      aliases:
+        - schema
       suboptions:
         extra_data:
           description:
@@ -261,7 +287,7 @@ options:
             type:
               description:
                 - Name of unified job template type to run in the workflow.
-                - Can be a job_template, project, inventory_source, workflow_approval.
+                - Can be a job_template, project, inventory_source, system_job_template, workflow_approval, workflow_job_template.
               type: str
             timeout:
               description:
@@ -316,29 +342,31 @@ options:
                   description:
                     - Name Credentials to be applied to job as launch-time prompts.
                   elements: str
-    destroy_current_schema:
+    destroy_current_nodes:
       description:
-        - Set in order to destroy current schema on the workflow.
-        - This option is used for full schema update, if not used, nodes not described in schema will persist and keep current associations and links.
+        - Set in order to destroy current workflow_nodes on the workflow.
+        - This option is used for full workflow update, if not used, nodes not described in workflow will persist and keep current associations and links.
       type: bool
       default: False
+      aliases:
+        - destroy_current_schema
 
 extends_documentation_fragment: awx.awx.auth
 '''
 
 EXAMPLES = '''
 - name: Create a workflow job template
-  tower_workflow_job_template:
+  workflow_job_template:
     name: example-workflow
     description: created by Ansible Playbook
     organization: Default
 
-- name: Create a workflow job template with schema in template
-  awx.awx.tower_workflow_job_template:
+- name: Create a workflow job template with workflow nodes in template
+  awx.awx.workflow_job_template:
     name: example-workflow
     inventory: Demo Inventory
     extra_vars: {'foo': 'bar', 'another-foo': {'barz': 'bar2'}}
-    schema:
+    workflow_nodes:
       - identifier: node101
         unified_job_template:
           name: example-project
@@ -391,17 +419,17 @@ EXAMPLES = '''
   register: result
 
 - name: Copy a workflow job template
-  tower_workflow_job_template:
+  workflow_job_template:
     name: copy-workflow
     copy_from: example-workflow
     organization: Foo
 
-- name: Create a workflow job template with schema in template
-  awx.awx.tower_workflow_job_template:
+- name: Create a workflow job template with workflow nodes in template
+  awx.awx.workflow_job_template:
     name: example-workflow
     inventory: Demo Inventory
     extra_vars: {'foo': 'bar', 'another-foo': {'barz': 'bar2'}}
-    schema:
+    workflow_nodes:
       - identifier: node101
         unified_job_template:
           name: example-project
@@ -455,7 +483,7 @@ EXAMPLES = '''
 
 '''
 
-from ..module_utils.tower_api import TowerAPIModule
+from ..module_utils.controller_api import ControllerAPIModule
 
 import json
 
@@ -477,28 +505,30 @@ def update_survey(module, last_request):
             module.fail_json(msg="Failed to update survey: {0}".format(response['json']['error']))
 
 
-def create_schema_nodes(module, response, schema, workflow_id):
-    for workflow_node in schema:
+def create_workflow_nodes(module, response, workflow_nodes, workflow_id):
+    for workflow_node in workflow_nodes:
         workflow_node_fields = {}
         search_fields = {}
         association_fields = {}
 
         # Lookup Job Template ID
         if workflow_node['unified_job_template']['name']:
-            search_fields = {'name': workflow_node['unified_job_template']['name']}
             if workflow_node['unified_job_template']['type'] is None:
-                module.fail_json(msg='Could not find unified job template type in schema {1}'.format(workflow_node))
+                module.fail_json(msg='Could not find unified job template type in workflow_nodes {1}'.format(workflow_node))
+            search_fields['type'] = workflow_node['unified_job_template']['type']
             if workflow_node['unified_job_template']['type'] == 'inventory_source':
-                # workflow_node['unified_job_template']['inventory']:
-                organization_id = module.resolve_name_to_id('organizations', workflow_node['unified_job_template']['inventory']['organization']['name'])
-                search_fields['organization'] = organization_id
-            elif workflow_node['unified_job_template']['type'] == 'workflow_approval':
-                pass
-            else:
-                # workflow_node['unified_job_template']['organization']:
+                if 'inventory' in workflow_node['unified_job_template']:
+                    if 'organization' in workflow_node['unified_job_template']['inventory']:
+                        organization_id = module.resolve_name_to_id('organizations', workflow_node['unified_job_template']['inventory']['organization']['name'])
+                        search_fields['organization'] = organization_id
+                    else:
+                        pass
+            elif 'organization' in workflow_node['unified_job_template']:
                 organization_id = module.resolve_name_to_id('organizations', workflow_node['unified_job_template']['organization']['name'])
                 search_fields['organization'] = organization_id
-            unified_job_template = module.get_one('unified_job_templates', **{'data': search_fields})
+            else:
+                pass
+            unified_job_template = module.get_one('unified_job_templates', name_or_id=workflow_node['unified_job_template']['name'], **{'data': search_fields})
             if unified_job_template:
                 workflow_node_fields['unified_job_template'] = unified_job_template['id']
             else:
@@ -558,8 +588,6 @@ def create_schema_nodes(module, response, schema, workflow_id):
 
         # Start Approval Node creation process
         if workflow_node['unified_job_template']['type'] == 'workflow_approval':
-            new_fields = {}
-
             for field_name in (
                 'name',
                 'description',
@@ -588,8 +616,8 @@ def create_schema_nodes(module, response, schema, workflow_id):
             )
 
 
-def create_schema_nodes_association(module, response, schema, workflow_id):
-    for workflow_node in schema:
+def create_workflow_nodes_association(module, response, workflow_nodes, workflow_id):
+    for workflow_node in workflow_nodes:
         workflow_node_fields = {}
         search_fields = {}
         association_fields = {}
@@ -630,7 +658,6 @@ def create_schema_nodes_association(module, response, schema, workflow_id):
                         if sub_obj is None:
                             module.fail_json(msg='Could not find {0} entry with name {1}'.format(association, sub_name))
                         id_list.append(sub_obj['id'])
-                        temp = sub_obj['id']
                     if id_list:
                         association_fields[association] = id_list
 
@@ -644,7 +671,7 @@ def create_schema_nodes_association(module, response, schema, workflow_id):
                     )
 
 
-def destroy_schema_nodes(module, response, workflow_id):
+def destroy_workflow_nodes(module, response, workflow_id):
     search_fields = {}
 
     # Search for existing nodes.
@@ -664,12 +691,15 @@ def main():
         copy_from=dict(),
         description=dict(),
         extra_vars=dict(type='dict'),
+        job_tags=dict(),
+        skip_tags=dict(),
         organization=dict(),
-        execution_environment=dict(),
         survey_spec=dict(type='dict', aliases=['survey']),
         survey_enabled=dict(type='bool'),
         allow_simultaneous=dict(type='bool'),
         ask_variables_on_launch=dict(type='bool'),
+        ask_labels_on_launch=dict(type='bool', aliases=['ask_labels']),
+        ask_skip_tags_on_launch=dict(type='bool', aliases=['ask_skip_tags']),
         inventory=dict(),
         limit=dict(),
         scm_branch=dict(),
@@ -683,13 +713,13 @@ def main():
         notification_templates_success=dict(type="list", elements='str'),
         notification_templates_error=dict(type="list", elements='str'),
         notification_templates_approvals=dict(type="list", elements='str'),
-        schema=dict(type='list', elements='dict'),
-        destroy_current_schema=dict(type='bool', default=False),
+        workflow_nodes=dict(type='list', elements='dict', aliases=['schema']),
+        destroy_current_nodes=dict(type='bool', default=False, aliases=['destroy_current_schema']),
         state=dict(choices=['present', 'absent'], default='present'),
     )
 
     # Create a module for ourselves
-    module = TowerAPIModule(argument_spec=argument_spec)
+    module = ControllerAPIModule(argument_spec=argument_spec)
 
     # Extract our parameters
     name = module.params.get('name')
@@ -698,10 +728,10 @@ def main():
     state = module.params.get('state')
 
     # Extract schema parameters
-    schema = None
-    if module.params.get('schema'):
-        schema = module.params.get('schema')
-    destroy_current_schema = module.params.get('destroy_current_schema')
+    workflow_nodes = None
+    if module.params.get('workflow_nodes'):
+        workflow_nodes = module.params.get('workflow_nodes')
+    destroy_current_nodes = module.params.get('destroy_current_nodes')
 
     new_fields = {}
     search_fields = {}
@@ -712,10 +742,6 @@ def main():
         organization_id = module.resolve_name_to_id('organizations', organization)
         search_fields['organization'] = new_fields['organization'] = organization_id
 
-    ee = module.params.get('execution_environment')
-    if ee:
-        new_fields['execution_environment'] = module.resolve_name_to_id('execution_environments', ee)
-
     # Attempt to look up an existing item based on the provided data
     existing_item = module.get_one('workflow_job_templates', name_or_id=name, **{'data': search_fields})
 
@@ -741,7 +767,7 @@ def main():
 
     webhook_credential = module.params.get('webhook_credential')
     if webhook_credential:
-        new_fields['webhook_credential'] = module.resolve_name_to_id('webhook_credential', webhook_credential)
+        new_fields['webhook_credential'] = module.resolve_name_to_id('credentials', webhook_credential)
 
     # Create the data that gets sent for create and update
     new_fields['name'] = new_name if new_name else (module.get_item_name(existing_item) if existing_item else name)
@@ -756,10 +782,14 @@ def main():
         'ask_scm_branch_on_launch',
         'ask_limit_on_launch',
         'ask_variables_on_launch',
+        'ask_labels_on_launch',
+        'ask_skip_tags_on_launch',
         'webhook_service',
+        'job_tags',
+        'skip_tags',
     ):
         field_val = module.params.get(field_name)
-        if field_val:
+        if field_val is not None:
             new_fields[field_name] = field_val
 
     if 'extra_vars' in new_fields:
@@ -796,7 +826,10 @@ def main():
         association_fields['labels'] = []
         for item in labels:
             label_id = module.get_one('labels', name_or_id=item, **{'data': search_fields})
-            association_fields['labels'].append(label_id['id'])
+            if label_id is None:
+                module.fail_json(msg='Could not find label entry with name {0}'.format(item))
+            else:
+                association_fields['labels'].append(label_id['id'])
 
     on_change = None
     new_spec = module.params.get('survey_spec')
@@ -824,19 +857,19 @@ def main():
     )
 
     # Get Workflow information in case one was just created.
-    existing_item = module.get_one('workflow_job_templates', name_or_id=name, **{'data': search_fields})
+    existing_item = module.get_one('workflow_job_templates', name_or_id=new_name if new_name else name, **{'data': search_fields})
     workflow_job_template_id = existing_item['id']
     # Destroy current nodes if selected.
-    if destroy_current_schema:
-        destroy_schema_nodes(module, response, workflow_job_template_id)
+    if destroy_current_nodes:
+        destroy_workflow_nodes(module, response, workflow_job_template_id)
 
     # Work thorugh and lookup value for schema fields
-    if schema:
+    if workflow_nodes:
         # Create Schema Nodes
-        create_schema_nodes(module, response, schema, workflow_job_template_id)
+        create_workflow_nodes(module, response, workflow_nodes, workflow_job_template_id)
         # Create Schema Associations
-        create_schema_nodes_association(module, response, schema, workflow_job_template_id)
-        module.json_output['schema_creation_data'] = response
+        create_workflow_nodes_association(module, response, workflow_nodes, workflow_job_template_id)
+        module.json_output['node_creation_data'] = response
 
     module.exit_json(**module.json_output)
 
