@@ -7,28 +7,35 @@
 DOCUMENTATION = r"""
 ---
 module: sg_to_vn_to_vlan_bulk_request
-short_description: Resource module for Sg To Vn To Vlan Bulk Request
+short_description: Resource module for SG To VN To VLAN Bulk Request
 description:
-- Manage operation update of the resource Sg To Vn To Vlan Bulk Request.
+- Manage operation update of the resource SG To VN To VLAN Bulk Request.
+- This API allows the client to submit the bulk request.
 version_added: '1.0.0'
 extends_documentation_fragment:
   - cisco.ise.module
 author: Rafael Campos (@racampos)
 options:
   operationType:
-    description: Sg To Vn To Vlan Bulk Request's operationType.
+    description: SG To VN To VLAN Bulk Request's operationType.
     type: str
   resourceMediaType:
-    description: Sg To Vn To Vlan Bulk Request's resourceMediaType.
+    description: SG To VN To VLAN Bulk Request's resourceMediaType.
     type: str
 requirements:
-- ciscoisesdk >= 1.1.0
+- ciscoisesdk >= 2.0.3
 - python >= 3.5
 seealso:
-# Reference by Internet resource
-- name: Sg To Vn To Vlan Bulk Request reference
-  description: Complete reference of the Sg To Vn To Vlan Bulk Request object model.
-  link: https://ciscoisesdk.readthedocs.io/en/latest/api/api.html#v3-0-0-summary
+- name: Cisco ISE documentation for SecurityGroupToVirtualNetwork
+  description: Complete reference of the SecurityGroupToVirtualNetwork API.
+  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!sgtvnvlan
+notes:
+  - SDK Method used are
+    security_group_to_virtual_network.SecurityGroupToVirtualNetwork.bulk_request_for_security_groups_to_vn_to_vlan,
+
+  - Paths used are
+    put /ers/config/sgtvnvlan/bulk/submit,
+
 """
 
 EXAMPLES = r"""
