@@ -19,11 +19,6 @@ description:
     - Allows for easy creation, updating and deletion of origin access
       identities.
 
-requirements:
-  - boto3 >= 1.0.0
-  - python >= 2.6
-
-
 author: Willem van Ketwich (@wilvk)
 
 extends_documentation_fragment:
@@ -261,8 +256,7 @@ def main():
         else:
             result = service_mgr.create_origin_access_identity(caller_reference, comment)
             changed = True
-    elif(state == 'absent' and origin_access_identity_id is not None and
-         e_tag is not None):
+    elif state == 'absent' and origin_access_identity_id is not None and e_tag is not None:
         result = service_mgr.delete_origin_access_identity(origin_access_identity_id, e_tag)
         changed = True
 
