@@ -1,8 +1,10 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
+
 # Copyright: (c) 2018, Ansible Project
 # Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
-# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
+# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 from __future__ import absolute_import, division, print_function
 __metaclass__ = type
@@ -19,11 +21,6 @@ description:
     - Be careful while removing disk specified as this may lead to data loss.
 author:
     - Abhijeet Kasurde (@Akasurde) <akasurde@redhat.com>
-notes:
-    - Tested on vSphere 6.0 and 6.5
-requirements:
-    - "python >= 2.6"
-    - PyVmomi
 options:
    name:
      description:
@@ -659,6 +656,7 @@ class PyVmomiHelper(PyVmomi):
         disk_change_list = list()
         results = dict(changed=False, disk_data=None, disk_changes=dict())
         new_added_disk_ctl = list()
+        sharesval = {'low': 500, 'normal': 1000, 'high': 2000}
 
         # Deal with controller
         for disk in disk_data:
@@ -699,6 +697,7 @@ class PyVmomiHelper(PyVmomi):
         # Deal with Disks
         for disk in disk_data:
             disk_found = False
+            update_io = False
             disk_change = False
             ctl_found = False
             for device in self.vm.config.hardware.device:
@@ -709,8 +708,27 @@ class PyVmomiHelper(PyVmomi):
                             disk_found = True
                             if disk['state'] == 'present':
                                 disk_spec = vim.vm.device.VirtualDeviceSpec()
-                                # set the operation to edit so that it knows to keep other settings
                                 disk_spec.device = disk_device
+                                # Deal with iolimit. Note that if iolimit is set, you HAVE TO both set limit and shares,
+                                #  or ansible will break with "'NoneType' object is not subscriptable"
+                                if 'iolimit' in disk:
+                                    if disk['iolimit']['limit'] != disk_spec.device.storageIOAllocation.limit:
+                                        update_io = True
+
+                                    if 'shares' in disk['iolimit']:
+                                        # 'low', 'normal' and 'high' values in disk['iolimit']['shares']['level'] are converted to int values on vcenter side
+                                        if (disk['iolimit']['shares']['level'] != 'custom'
+                                            and sharesval.get(disk['iolimit']['shares']['level'], 0) != disk_spec.device.storageIOAllocation.shares.shares) or \
+                                            (disk['iolimit']['shares']['level'] == 'custom'
+                                                and disk['iolimit']['shares']['level_value'] != disk_spec.device.storageIOAllocation.shares.shares):
+                                            update_io = True
+
+                                    if update_io:
+                                        # set the operation to edit so that it knows to keep other settings
+                                        disk_spec.operation = vim.vm.device.VirtualDeviceSpec.Operation.edit
+                                        disk_spec = self.get_ioandshares_diskconfig(disk_spec, disk)
+                                        disk_change = True
+
                                 # If this is an RDM ignore disk size
                                 if disk['disk_type'] != 'rdm':
                                     if disk['size'] < disk_spec.device.capacityInKB:
@@ -719,14 +737,17 @@ class PyVmomiHelper(PyVmomi):
                                                                   % (disk['disk_index'], disk['size'],
                                                                       disk_spec.device.capacityInKB))
                                     if disk['size'] != disk_spec.device.capacityInKB:
+                                        # set the operation to edit so that it knows to keep other settings
                                         disk_spec.operation = vim.vm.device.VirtualDeviceSpec.Operation.edit
                                         if disk['disk_type'] != 'vpmemdisk':
                                             disk_spec = self.get_ioandshares_diskconfig(disk_spec, disk)
                                         disk_spec.device.capacityInKB = disk['size']
-                                        self.config_spec.deviceChange.append(disk_spec)
                                         disk_change = True
-                                        disk_change_list.append(disk_change)
-                                        results['disk_changes'][disk['disk_index']] = "Disk reconfigured."
+
+                                if disk_change:
+                                    self.config_spec.deviceChange.append(disk_spec)
+                                    disk_change_list.append(disk_change)
+                                    results['disk_changes'][disk['disk_index']] = "Disk reconfigured."
 
                             elif disk['state'] == 'absent':
                                 # Disk already exists, deleting
