@@ -15,7 +15,6 @@ import json
 from itertools import chain
 
 from ansible.module_utils.common.text.converters import to_text
-
 from ansible.module_utils._text import to_native
 from ansible.module_utils.common.collections import is_iterable
 from ansible.module_utils.basic import AnsibleModule, missing_required_lib, _load_params
@@ -33,7 +32,13 @@ except ImportError:
 
 # Used to map endpoints to applications dynamically
 API_APPS_ENDPOINTS = dict(
-    circuits=["circuits", "circuit_types", "circuit_terminations", "providers"],
+    circuits=[
+        "circuits",
+        "circuit_types",
+        "circuit_terminations",
+        "providers",
+        "provider_networks",
+    ],
     dcim=[
         "cables",
         "console_ports",
@@ -69,7 +74,14 @@ API_APPS_ENDPOINTS = dict(
         "site_groups",
         "virtual_chassis",
     ],
-    extras=["config_contexts", "tags"],
+    extras=[
+        "config_contexts",
+        "tags",
+        "custom_fields",
+        "custom_links",
+        "export_templates",
+        "webhooks",
+    ],
     ipam=[
         "aggregates",
         "ip_addresses",
@@ -83,8 +95,14 @@ API_APPS_ENDPOINTS = dict(
         "services",
     ],
     secrets=[],
-    tenancy=["tenants", "tenant_groups"],
-    virtualization=["cluster_groups", "cluster_types", "clusters", "virtual_machines"],
+    tenancy=["tenants", "tenant_groups", "contacts", "contact_groups", "contact_roles"],
+    virtualization=[
+        "cluster_groups",
+        "cluster_types",
+        "clusters",
+        "virtual_machines",
+    ],
+    wireless=["wireless_lans", "wireless_lan_groups", "wireless_links"],
 )
 
 # Used to normalize data for the respective query types used to find endpoints
@@ -96,10 +114,15 @@ QUERY_TYPES = dict(
     cluster_group="slug",
     cluster_type="slug",
     config_context="name",
+    contact_group="name",
+    contact_role="name",
+    custom_field="name",
+    custom_link="name",
     device="name",
     device_role="slug",
     device_type="slug",
     export_targets="name",
+    export_template="name",
     group="slug",
     installed_device="name",
     import_targets="name",
@@ -107,10 +130,12 @@ QUERY_TYPES = dict(
     manufacturer="slug",
     nat_inside="address",
     nat_outside="address",
+    parent_contact_group="name",
     parent_location="slug",
     parent_region="slug",
     parent_site_group="slug",
     parent_tenant_group="slug",
+    parent_wireless_lan_group="slug",
     power_panel="name",
     power_port="name",
     power_port_template="name",
@@ -120,6 +145,7 @@ QUERY_TYPES = dict(
     primary_ip4="address",
     primary_ip6="address",
     provider="slug",
+    provider_network="name",
     rack="name",
     rack_group="slug",
     rack_role="slug",
@@ -141,20 +167,27 @@ QUERY_TYPES = dict(
     vlan_group="slug",
     vlan_role="name",
     vrf="name",
+    webhook="name",
+    wireless_lan="ssid",
+    wireless_lan_group="slug",
 )
 
 # Specifies keys within data that need to be converted to ID and the endpoint to be used when queried
 CONVERT_TO_ID = {
     "assigned_object": "assigned_object",
+    "bridge": "interfaces",
     "circuit": "circuits",
     "circuit_type": "circuit_types",
     "circuit_termination": "circuit_terminations",
     "circuits.circuittermination": "circuit_terminations",
     "cluster": "clusters",
+    "clusters": "clusters",
     "cluster_group": "cluster_groups",
     "cluster_groups": "cluster_groups",
     "cluster_type": "cluster_types",
+    "cluster_types": "cluster_types",
     "config_context": "config_contexts",
+    "contact_groups": "contact_groups",
     "dcim.consoleport": "console_ports",
     "dcim.consoleserverport": "console_server_ports",
     "dcim.frontport": "front_ports",
@@ -172,6 +205,8 @@ CONVERT_TO_ID = {
     "import_targets": "route_targets",
     "installed_device": "devices",
     "interface": "interfaces",
+    "interface_a": "interfaces",
+    "interface_b": "interfaces",
     "interface_template": "interface_templates",
     "ip_addresses": "ip_addresses",
     "ipaddresses": "ip_addresses",
@@ -182,12 +217,15 @@ CONVERT_TO_ID = {
     "nat_inside": "ip_addresses",
     "nat_outside": "ip_addresses",
     "platform": "platforms",
+    "parent_contact_group": "contact_groups",
     "parent_location": "locations",
     "parent_interface": "interfaces",
+    "parent_inventory_item": "inventory_items",
     "parent_vm_interface": "interfaces",
     "parent_region": "regions",
     "parent_site_group": "site_groups",
     "parent_tenant_group": "tenant_groups",
+    "parent_wireless_lan_group": "wireless_lan_groups",
     "platforms": "platforms",
     "power_panel": "power_panels",
     "power_port": "power_ports",
@@ -197,6 +235,7 @@ CONVERT_TO_ID = {
     "primary_ip4": "ip_addresses",
     "primary_ip6": "ip_addresses",
     "provider": "providers",
+    "provider_network": "provider_networks",
     "rack": "racks",
     "rack_group": "rack_groups",
     "rack_role": "rack_roles",
@@ -211,6 +250,7 @@ CONVERT_TO_ID = {
     "scope": "sites",
     "services": "services",
     "site": "sites",
+    "site_groups": "site_groups",
     "site_group": "site_groups",
     "sites": "sites",
     "tags": "tags",
@@ -225,10 +265,14 @@ CONVERT_TO_ID = {
     "virtual_chassis": "virtual_chassis",
     "virtual_machine": "virtual_machines",
     "virtual_machine_role": "device_roles",
+    "vm_bridge": "interfaces",
     "vlan": "vlans",
     "vlan_group": "vlan_groups",
     "vlan_role": "roles",
     "vrf": "vrfs",
+    "wireless_lan": "wireless_lans",
+    "wireless_lan_group": "wireless_lan_groups",
+    "wireless_link": "wireless_links",
 }
 
 ENDPOINT_NAME_MAPPING = {
@@ -245,11 +289,17 @@ ENDPOINT_NAME_MAPPING = {
     "console_port_templates": "console_port_template",
     "console_server_ports": "console_server_port",
     "console_server_port_templates": "console_server_port_template",
+    "contacts": "contact",
+    "contact_groups": "contact_group",
+    "contact_roles": "contact_role",
+    "custom_fields": "custom_field",
+    "custom_links": "custom_link",
     "device_bays": "device_bay",
     "device_bay_templates": "device_bay_template",
     "devices": "device",
     "device_roles": "device_role",
     "device_types": "device_type",
+    "export_templates": "export_template",
     "front_ports": "front_port",
     "front_port_templates": "front_port_template",
     "interfaces": "interface",
@@ -267,6 +317,7 @@ ENDPOINT_NAME_MAPPING = {
     "power_port_templates": "power_port_template",
     "prefixes": "prefix",
     "providers": "provider",
+    "provider_networks": "provider_network",
     "racks": "rack",
     "rack_groups": "rack_group",
     "rack_roles": "rack_role",
@@ -287,11 +338,16 @@ ENDPOINT_NAME_MAPPING = {
     "vlans": "vlan",
     "vlan_groups": "vlan_group",
     "vrfs": "vrf",
+    "webhooks": "webhook",
+    "wireless_lans": "wireless_lan",
+    "wireless_lan_groups": "wireless_lan_group",
+    "wireless_links": "wireless_link",
 }
 
 ALLOWED_QUERY_PARAMS = {
     "aggregate": set(["prefix", "rir"]),
     "assigned_object": set(["name", "device", "virtual_machine"]),
+    "bridge": set(["name", "device"]),
     "circuit": set(["cid"]),
     "circuit_type": set(["slug"]),
     "circuit_termination": set(["circuit", "term_side"]),
@@ -303,10 +359,12 @@ ALLOWED_QUERY_PARAMS = {
         [
             "name",
             "regions",
+            "site_groups",
             "sites",
             "roles",
             "device_types",
             "platforms",
+            "cluster_types",
             "cluster_groups",
             "clusters",
             "tenant_groups",
@@ -318,6 +376,11 @@ ALLOWED_QUERY_PARAMS = {
     "console_port_template": set(["name", "device_type"]),
     "console_server_port": set(["name", "device"]),
     "console_server_port_template": set(["name", "device_type"]),
+    "contact": set(["name", "group"]),
+    "contact_group": set(["name"]),
+    "contact_role": set(["name"]),
+    "custom_field": set(["name"]),
+    "custom_link": set(["name"]),
     "dcim.consoleport": set(["name", "device"]),
     "dcim.consoleserverport": set(["name", "device"]),
     "dcim.frontport": set(["name", "device", "rear_port"]),
@@ -331,22 +394,29 @@ ALLOWED_QUERY_PARAMS = {
     "device": set(["name"]),
     "device_role": set(["slug"]),
     "device_type": set(["slug"]),
+    "export_template": set(["name"]),
     "front_port": set(["name", "device", "rear_port"]),
     "front_port_template": set(["name", "device_type", "rear_port"]),
     "installed_device": set(["name"]),
     "interface": set(["name", "device", "virtual_machine"]),
+    "interface_a": set(["name", "device"]),
+    "interface_b": set(["name", "device"]),
     "interface_template": set(["name", "device_type"]),
     "inventory_item": set(["name", "device"]),
     "ip_address": set(["address", "vrf", "device", "interface", "assigned_object"]),
     "ip_addresses": set(["address", "vrf", "device", "interface", "assigned_object"]),
-    "ipaddresses": set(["address", "vrf", "device", "interface", "assigned_object"]),
+    "ipaddresses": set(
+        ["address", "vrf", "device", "interface", "assigned_object", "virtual_machine"]
+    ),
     "lag": set(["name"]),
     "location": set(["slug"]),
     "manufacturer": set(["slug"]),
     "master": set(["name"]),
     "nat_inside": set(["vrf", "address"]),
+    "parent_contact_group": set(["name"]),
     "parent_location": set(["slug"]),
     "parent_interface": set(["name"]),
+    "parent_inventory_item": set(["name", "device"]),
     "parent_vm_interface": set(["name"]),
     "parent_region": set(["slug"]),
     "parent_site_group": set(["slug"]),
@@ -362,7 +432,8 @@ ALLOWED_QUERY_PARAMS = {
     "primary_ip4": set(["address", "vrf"]),
     "primary_ip6": set(["address", "vrf"]),
     "provider": set(["slug"]),
-    "rack": set(["name", "site"]),
+    "provider_network": set(["name"]),
+    "rack": set(["name", "site", "location"]),
     "rack_group": set(["slug"]),
     "rack_role": set(["slug"]),
     "region": set(["slug"]),
@@ -372,7 +443,7 @@ ALLOWED_QUERY_PARAMS = {
     "role": set(["slug"]),
     "route_target": set(["name"]),
     "services": set(["device", "virtual_machine", "name", "port", "protocol"]),
-    "site": set(["slug"]),
+    "site": set(["slug", "name"]),
     "site_group": set(["slug"]),
     "tags": set(["slug"]),
     "tagged_vlans": set(["group", "name", "site", "vid", "vlan_group", "tenant"]),
@@ -383,9 +454,14 @@ ALLOWED_QUERY_PARAMS = {
     "untagged_vlan": set(["group", "name", "site", "vid", "vlan_group", "tenant"]),
     "virtual_chassis": set(["name", "master"]),
     "virtual_machine": set(["name", "cluster"]),
+    "vm_bridge": set(["name"]),
     "vlan": set(["group", "name", "site", "tenant", "vid", "vlan_group"]),
-    "vlan_group": set(["slug", "site", "scope"]),
+    "vlan_group": set(["name", "slug", "site", "scope"]),
     "vrf": set(["name", "tenant"]),
+    "webhook": set(["name"]),
+    "wireless_lan": set(["ssid"]),
+    "wireless_lan_group": set(["name"]),
+    "wireless_link": set(["interface_a", "interface_b"]),
 }
 
 QUERY_PARAMS_IDS = set(
@@ -395,6 +471,7 @@ QUERY_PARAMS_IDS = set(
         "device",
         "group",
         "interface",
+        "location",
         "rir",
         "vrf",
         "site",
@@ -435,19 +512,23 @@ REQUIRED_ID_FIND = {
     "vlans": set(["status"]),
 }
 
-# This is used to map non-clashing keys to Netbox API compliant keys to prevent bad logic in code for similar keys but different modules
+# This is used to map non-clashing keys to NetBox API compliant keys to prevent bad logic in code for similar keys but different modules
 CONVERT_KEYS = {
     "assigned_object": "assigned_object_id",
     "scope": "scope_id",
     "circuit_type": "type",
     "cluster_type": "type",
     "cluster_group": "group",
+    "contact_group": "group",
+    "parent_contact_group": "parent",
     "parent_location": "parent",
     "parent_interface": "parent",
+    "parent_inventory_item": "parent",
     "parent_vm_interface": "parent",
     "parent_region": "parent",
     "parent_site_group": "parent",
     "parent_tenant_group": "parent",
+    "parent_wireless_lan_group": "parent",
     "power_port_template": "power_port",
     "prefix_role": "role",
     "rack_group": "group",
@@ -461,6 +542,8 @@ CONVERT_KEYS = {
     "virtual_machine_role": "role",
     "vlan_role": "role",
     "vlan_group": "group",
+    "vm_bridge": "bridge",
+    "wireless_lan_group": "group",
 }
 
 # This is used to dynamically convert name to slug on endpoints requiring a slug
@@ -468,6 +551,8 @@ SLUG_REQUIRED = {
     "circuit_types",
     "cluster_groups",
     "cluster_types",
+    "contact_groups",
+    "contact_roles",
     "device_roles",
     "device_types",
     "ipam_roles",
@@ -486,6 +571,7 @@ SLUG_REQUIRED = {
     "platforms",
     "providers",
     "vlan_groups",
+    "wireless_lan_groups",
 }
 
 SCOPE_TO_ENDPOINT = {
@@ -510,7 +596,7 @@ NETBOX_ARG_SPEC = dict(
 
 class NetboxModule(object):
     """
-    Initialize connection to Netbox, sets AnsibleModule passed in to
+    Initialize connection to NetBox, sets AnsibleModule passed in to
     self.module to be used throughout the class
     :params module (obj): Ansible Module object
     :params endpoint (str): Used to tell class which endpoint the logic needs to follow
@@ -528,20 +614,24 @@ class NetboxModule(object):
             self.module.fail_json(
                 msg=missing_required_lib("pynetbox"), exception=PYNETBOX_IMP_ERR
             )
-        # These should not be required after making connection to Netbox
+        # These should not be required after making connection to NetBox
         url = self.module.params["netbox_url"]
         token = self.module.params["netbox_token"]
         ssl_verify = self.module.params["validate_certs"]
         cert = self.module.params["cert"]
 
-        # Attempt to initiate connection to Netbox
+        # Attempt to initiate connection to NetBox
         if nb_client is None:
             self.nb = self._connect_netbox_api(url, token, ssl_verify, cert)
         else:
             self.nb = nb_client
             try:
                 self.version = self.nb.version
-                self.full_version = self.nb.status().get("netbox-version")
+                try:
+                    self.full_version = self.nb.status().get("netbox-version")
+                except Exception:
+                    # For NetBox versions without /api/status endpoint
+                    self.full_version = f"{self.version}.0"
             except AttributeError:
                 self.module.fail_json(msg="Must have pynetbox >=4.1.0")
 
@@ -590,16 +680,20 @@ class NetboxModule(object):
             nb.http_session = session
             try:
                 self.version = nb.version
-                self.full_version = nb.status().get("netbox-version")
+                try:
+                    self.full_version = nb.status().get("netbox-version")
+                except Exception:
+                    # For NetBox versions without /api/status endpoint
+                    self.full_version = f"{self.version}.0"
             except AttributeError:
                 self.module.fail_json(msg="Must have pynetbox >=4.1.0")
             except Exception:
                 self.module.fail_json(
-                    msg="Failed to establish connection to Netbox API"
+                    msg="Failed to establish connection to NetBox API"
                 )
             return nb
         except Exception:
-            self.module.fail_json(msg="Failed to establish connection to Netbox API")
+            self.module.fail_json(msg="Failed to establish connection to NetBox API")
 
     def _nb_endpoint_get(self, nb_endpoint, query_params, search_item):
         try:
@@ -669,7 +763,7 @@ class NetboxModule(object):
         """
         Used to change non-clashing keys for each module into identical keys that are required
         to be passed to pynetbox
-        ex. rack_role back into role to pass to Netbox
+        ex. rack_role back into role to pass to NetBox
         Returns data
         :params data (dict): Data dictionary after _find_ids method ran
         """
@@ -732,10 +826,10 @@ class NetboxModule(object):
     ):
         """
         :returns dict(query_dict): Returns a query dictionary built using mappings to dynamically
-        build available query params for Netbox endpoints
+        build available query params for NetBox endpoints
         :params parent(str): This is either a key from `_find_ids` or a string passed in to determine
         which keys in the data that we need to use to construct `query_dict`
-        :params module_data(dict): Uses the data provided to the Netbox module
+        :params module_data(dict): Uses the data provided to the NetBox module
         :params child(dict): This is used within `_find_ids` and passes the inner dictionary
         to build the appropriate `query_dict` for the parent
         """
@@ -764,7 +858,11 @@ class NetboxModule(object):
                     query_id = self._get_query_param_id(match, child)
                 else:
                     query_id = self._get_query_param_id(match, module_data)
-                query_dict.update({match + "_id": query_id})
+
+                if parent == "vlan_group" and match == "site":
+                    query_dict.update({match: query_id})
+                else:
+                    query_dict.update({match + "_id": query_id})
             else:
                 if child:
                     value = child.get(match)
@@ -793,6 +891,11 @@ class NetboxModule(object):
             if not child:
                 query_dict["name"] = module_data["parent_vm_interface"]
 
+        elif parent == "vm_bridge" and module_data.get("virtual_machine"):
+            if not child:
+                query_dict["name"] = module_data["vm_bridge"]
+                query_dict["virtual_machine_id"] = module_data["virtual_machine"]
+
         elif parent == "lag":
             if not child:
                 query_dict["name"] = module_data["lag"]
@@ -826,7 +929,7 @@ class NetboxModule(object):
                 )
 
         elif parent == "virtual_chassis":
-            query_dict = {"q": self.module.params["data"].get("master")}
+            query_dict.update({"master": self.module.params["data"].get("master")})
 
         elif parent == "rear_port" and self.endpoint == "front_ports":
             if isinstance(module_data.get("rear_port"), str):
@@ -928,6 +1031,7 @@ class NetboxModule(object):
         for k, v in API_APPS_ENDPOINTS.items():
             if endpoint in v:
                 nb_app = k
+
         return nb_app
 
     def _find_ids(self, data, user_query_params):
@@ -978,7 +1082,7 @@ class NetboxModule(object):
                             "device_types",
                             "platforms",
                             "cluster_groups",
-                            "clusters",
+                            "contact_groups",
                             "tenant_groups",
                             "tenants",
                             "tags",
@@ -995,7 +1099,7 @@ class NetboxModule(object):
                             id_list.append(list_item)
                             continue
                         else:
-                            temp_dict = {QUERY_TYPES.get(k, "q"): search}
+                            temp_dict = {QUERY_TYPES.get(k, "q"): list_item}
 
                         query_id = self._nb_endpoint_get(nb_endpoint, temp_dict, k)
                         if query_id:
@@ -1026,6 +1130,12 @@ class NetboxModule(object):
                         query_params = self._build_query_params(
                             k, data, user_query_params
                         )
+                    elif k == "vm_bridge":
+                        nb_app = getattr(self.nb, "virtualization")
+                        nb_endpoint = getattr(nb_app, endpoint)
+                        query_params = self._build_query_params(
+                            k, data, user_query_params
+                        )
                     else:
                         query_params = {QUERY_TYPES.get(k, "q"): search}
                     query_id = self._nb_endpoint_get(nb_endpoint, query_params, k)
@@ -1057,10 +1167,10 @@ class NetboxModule(object):
 
     def _normalize_data(self, data):
         """
-        :returns data (dict): Normalized module data to formats accepted by Netbox searches
+        :returns data (dict): Normalized module data to formats accepted by NetBox searches
         such as changing from user specified value to slug
         ex. Test Rack -> test-rack
-        :params data (dict): Original data from Netbox module
+        :params data (dict): Original data from NetBox module
         """
         for k, v in data.items():
             if isinstance(v, dict):
@@ -1104,9 +1214,9 @@ class NetboxModule(object):
         return data
 
     def _create_netbox_object(self, nb_endpoint, data):
-        """Create a Netbox object.
+        """Create a NetBox object.
         :returns tuple(serialized_nb_obj, diff): tuple of the serialized created
-        Netbox object and the Ansible diff.
+        NetBox object and the Ansible diff.
         """
         if self.check_mode:
             nb_obj = data
@@ -1120,7 +1230,7 @@ class NetboxModule(object):
         return nb_obj, diff
 
     def _delete_netbox_object(self):
-        """Delete a Netbox object.
+        """Delete a NetBox object.
         :returns diff (dict): Ansible diff
         """
         if not self.check_mode:
@@ -1133,9 +1243,9 @@ class NetboxModule(object):
         return diff
 
     def _update_netbox_object(self, data):
-        """Update a Netbox object.
+        """Update a NetBox object.
         :returns tuple(serialized_nb_obj, diff): tuple of the serialized updated
-        Netbox object and the Ansible diff.
+        NetBox object and the Ansible diff.
         """
         serialized_nb_obj = self.nb_object.serialize()
         updated_obj = serialized_nb_obj.copy()
@@ -1144,6 +1254,24 @@ class NetboxModule(object):
             serialized_nb_obj["tags"] = set(serialized_nb_obj["tags"])
             updated_obj["tags"] = set(data["tags"])
 
+        # Ensure idempotency for site and virtual machine on version pre-3.0
+        version_pre_30 = self._version_check_greater("3.0", self.version)
+        if (
+            serialized_nb_obj.get("latitude")
+            and data.get("latitude")
+            and version_pre_30
+        ):
+            updated_obj["latitude"] = str(data["latitude"])
+        if (
+            serialized_nb_obj.get("longitude")
+            and data.get("longitude")
+            and version_pre_30
+        ):
+            updated_obj["longitude"] = str(data["longitude"])
+
+        if serialized_nb_obj.get("vcpus") and data.get("vcpus") and version_pre_30:
+            updated_obj["vcpus"] = "{0:.2f}".format(data["vcpus"])
+
         if serialized_nb_obj == updated_obj:
             return serialized_nb_obj, None
         else:
@@ -1222,11 +1350,11 @@ class NetboxAnsibleModule(AnsibleModule):
     """
     Creating this due to needing to override some functionality to provide required_together, required_if
     and will be able to override more in the future.
-    This is due to the Netbox modules having the module arguments within a key in the argument spec, using suboptions rather than
+    This is due to the NetBox modules having the module arguments within a key in the argument spec, using suboptions rather than
     having all module arguments within the regular argument spec.
 
-    Didn't want to change that functionality of the Netbox modules as its disruptive and we're required to send a specific payload
-    to the Netbox API
+    Didn't want to change that functionality of the NetBox modules as its disruptive and we're required to send a specific payload
+    to the NetBox API
     """
 
     def __init__(
@@ -1311,7 +1439,7 @@ class NetboxAnsibleModule(AnsibleModule):
         return results
 
     def _check_required_if(self, spec, param=None):
-        """ ensure that parameters which conditionally required are present """
+        """ensure that parameters which conditionally required are present"""
         if spec is None:
             return
 
